@@ -6,7 +6,7 @@
 import io
 
 
-from pypykatz.alsadecryptor.kerberosticket import KerberosTicket, KerberosTicketType
+from pypykatz.commons.kerberosticket import KerberosTicket, KerberosTicketType
 from pypykatz.alsadecryptor.package_commons import PackageDecryptor
 from pypykatz.alsadecryptor.win_datatypes import PLIST_ENTRY, PRTL_AVL_TABLE
 from pypykatz.commons.common import WindowsMinBuild
@@ -16,10 +16,12 @@ class KerberosCredential:
 		self.credtype = 'kerberos'
 		self.username = None
 		self.password = None
+		self.password_raw = b''
 		self.domainname = None
 		self.luid = None
 		self.tickets = []
 		self.pin = None
+		self.pin_raw = None
 		self.cardinfo = None
 		
 	def __str__(self):
@@ -28,8 +30,10 @@ class KerberosCredential:
 		t += '\t\tDomain: %s\n' % self.domainname
 		if self.password is not None:
 			t += '\t\tPassword: %s\n' % self.password
+			t += '\t\tpassword (hex)%s\n' % self.password_raw.hex()
 		if self.pin is not None:
 			t += '\t\tPIN: %s\n' % self.pin
+			t += '\t\tPIN (hex): %s\n' % self.pin_raw.hex()
 		if self.cardinfo is not None:
 			t += '\t\tCARDINFO: \n'
 			t += '\t\t\tCardName: %s\n' % self.cardinfo['CardName']
@@ -50,9 +54,11 @@ class KerberosCredential:
 		t['credtype'] = self.credtype
 		t['username'] = self.username
 		t['password'] = self.password
+		t['password_raw'] = self.password_raw
 		t['domainname'] = self.domainname
 		t['luid'] = self.luid
 		t['pin'] = self.pin
+		t['pin_raw'] = self.pin_raw
 		t['cardinfo'] = self.cardinfo
 		t['tickets'] = []
 		for ticket in self.tickets:
@@ -70,75 +76,73 @@ class KerberosDecryptor(PackageDecryptor):
 		self.current_ticket_type = None
 		self.current_cred = None
 
-	def find_first_entry(self):
-		position = self.find_signature('kerberos.dll',self.decryptor_template.signature)
-		ptr_entry_loc = self.reader.get_ptr_with_offset(position + self.decryptor_template.first_entry_offset)
-		ptr_entry = self.reader.get_ptr(ptr_entry_loc)
+	async def find_first_entry(self):
+		position = await self.find_signature('kerberos.dll',self.decryptor_template.signature)
+		ptr_entry_loc = await self.reader.get_ptr_with_offset(position + self.decryptor_template.first_entry_offset)
+		ptr_entry = await self.reader.get_ptr(ptr_entry_loc)
 		return ptr_entry, ptr_entry_loc
 	
-	def handle_ticket(self, kerberos_ticket):
+	async def handle_ticket(self, kerberos_ticket):
 		try:
-			kt = KerberosTicket.parse(kerberos_ticket, self.reader, self.decryptor_template.sysinfo, self.current_ticket_type)
+			#input(kerberos_ticket)
+			kt = await KerberosTicket.aparse(kerberos_ticket, self.reader, self.decryptor_template.sysinfo, self.current_ticket_type)
 			self.current_cred.tickets.append(kt)
 			#print(str(kt))
 		except Exception as e:
 			raise e
 	
-	def start(self):
+	async def start(self):
 		try:
-			entry_ptr_value, entry_ptr_loc = self.find_first_entry()
+			entry_ptr_value, entry_ptr_loc = await self.find_first_entry()
 		except Exception as e:
 			self.log('Failed to find structs! Reason: %s' % e)
 			return
 		
 		if self.sysinfo.buildnumber < WindowsMinBuild.WIN_VISTA.value:
-			self.reader.move(entry_ptr_loc)
+			await self.reader.move(entry_ptr_loc)
 			entry_ptr = await PLIST_ENTRY.load(self.reader)
-			self.walk_list(entry_ptr, self.process_session_elist)
+			await self.walk_list(entry_ptr, self.process_session_elist)
 		else:
 			result_ptr_list = []
-			self.reader.move(entry_ptr_value)
-			start_node = await PRTL_AVL_TABLE.load(self.reader).read(self.reader)
-			self.walk_avl(start_node.BalancedRoot.RightChild, result_ptr_list)
+			await self.reader.move(entry_ptr_value)
+			avl_table = await PRTL_AVL_TABLE.load(self.reader)
+			start_node = await avl_table.read(self.reader)
+			await self.walk_avl(start_node.BalancedRoot.RightChild, result_ptr_list)
 			
 			for ptr in result_ptr_list:
-				self.log_ptr(ptr, self.decryptor_template.kerberos_session_struct.__name__)
-				self.reader.move(ptr)
-				kerberos_logon_session = self.decryptor_template.kerberos_session_struct(self.reader)
-				self.process_session(kerberos_logon_session)
+				await self.log_ptr(ptr, self.decryptor_template.kerberos_session_struct.__name__)
+				await self.reader.move(ptr)
+				kerberos_logon_session = await self.decryptor_template.kerberos_session_struct.load(self.reader)
+				await self.process_session(kerberos_logon_session)
 
-	def process_session_elist(self, elist):
-		self.reader.move(elist.location)
-		self.reader.read_uint() #Flink do not remove this line!
-		self.reader.read_uint() #Blink do not remove this line!
-		kerberos_logon_session = self.decryptor_template.kerberos_session_struct(self.reader)
-		self.process_session(kerberos_logon_session)
+	async def process_session_elist(self, elist):
+		await self.reader.move(elist.location)
+		await self.reader.read_uint() #Flink do not remove this line!
+		await self.reader.read_uint() #Blink do not remove this line!
+		kerberos_logon_session = await self.decryptor_template.kerberos_session_struct.load(self.reader)
+		await self.process_session(kerberos_logon_session)
 
-	def process_session(self, kerberos_logon_session):
+	async def process_session(self, kerberos_logon_session):
 		self.current_cred = KerberosCredential()
 		self.current_cred.luid = kerberos_logon_session.LocallyUniqueIdentifier
 		
-		self.current_cred.username = kerberos_logon_session.credentials.UserName.read_string(self.reader)
-		self.current_cred.domainname = kerberos_logon_session.credentials.Domaine.read_string(self.reader)
-		if self.current_cred.username.endswith('$') is True:
-			self.current_cred.password, raw_dec = self.decrypt_password(kerberos_logon_session.credentials.Password.read_maxdata(self.reader), bytes_expected=True)
-			if self.current_cred.password is not None:
-				self.current_cred.password = self.current_cred.password.hex()
-		else:
-			self.current_cred.password, raw_dec = self.decrypt_password(kerberos_logon_session.credentials.Password.read_maxdata(self.reader))
+		self.current_cred.username = await kerberos_logon_session.credentials.UserName.read_string(self.reader)
+		self.current_cred.domainname = await kerberos_logon_session.credentials.Domaine.read_string(self.reader)
+		pwdata = await kerberos_logon_session.credentials.Password.read_maxdata(self.reader)
+		self.current_cred.password, self.current_cred.password_raw = self.decrypt_password(pwdata)
 		
 		if kerberos_logon_session.SmartcardInfos.value != 0:
-			csp_info = kerberos_logon_session.SmartcardInfos.read(self.reader, override_finaltype = self.decryptor_template.csp_info_struct)
-			pin_enc = csp_info.PinCode.read_maxdata(self.reader)
+			csp_info = await kerberos_logon_session.SmartcardInfos.read(self.reader, override_finaltype = self.decryptor_template.csp_info_struct)
+			pin_enc = await csp_info.PinCode.read_maxdata(self.reader)
 			self.current_cred.pin, raw_dec = self.decrypt_password(pin_enc)
 			if csp_info.CspDataLength != 0:
 				self.current_cred.cardinfo = csp_info.CspData.get_infos()
 
 		#### key list (still in session) this is not a linked list (thank god!)
 		if kerberos_logon_session.pKeyList.value != 0:
-			key_list = kerberos_logon_session.pKeyList.read(self.reader, override_finaltype = self.decryptor_template.keys_list_struct)
+			key_list = await kerberos_logon_session.pKeyList.read(self.reader, override_finaltype = self.decryptor_template.keys_list_struct)
 			#print(key_list.cbItem)
-			key_list.read(self.reader, self.decryptor_template.hash_password_struct)
+			await key_list.read(self.reader, self.decryptor_template.hash_password_struct)
 			for key in key_list.KeyEntries:
 				pass
 				### GOOD
@@ -204,19 +208,19 @@ class KerberosDecryptor(PackageDecryptor):
 				kerberos_logon_session.Tickets_1.Flink.value != kerberos_logon_session.Tickets_1.Flink.location and \
 					kerberos_logon_session.Tickets_1.Flink.value != kerberos_logon_session.Tickets_1.Flink.location - 4 :
 			self.current_ticket_type = KerberosTicketType.TGS
-			self.walk_list(kerberos_logon_session.Tickets_1.Flink, self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
+			await self.walk_list(kerberos_logon_session.Tickets_1.Flink, self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
 		
 		if kerberos_logon_session.Tickets_2.Flink.value != 0 and \
 				kerberos_logon_session.Tickets_2.Flink.value != kerberos_logon_session.Tickets_2.Flink.location and \
 					kerberos_logon_session.Tickets_2.Flink.value != kerberos_logon_session.Tickets_2.Flink.location - 4 :
 			self.current_ticket_type = KerberosTicketType.CLIENT
-			self.walk_list(kerberos_logon_session.Tickets_2.Flink,self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
+			await self.walk_list(kerberos_logon_session.Tickets_2.Flink,self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
 		
 		if kerberos_logon_session.Tickets_3.Flink.value != 0 and \
 				kerberos_logon_session.Tickets_3.Flink.value != kerberos_logon_session.Tickets_3.Flink.location and \
 					kerberos_logon_session.Tickets_3.Flink.value != kerberos_logon_session.Tickets_3.Flink.location - 4 :
 			self.current_ticket_type = KerberosTicketType.TGT
-			self.walk_list(kerberos_logon_session.Tickets_3.Flink,self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
+			await self.walk_list(kerberos_logon_session.Tickets_3.Flink,self.handle_ticket , override_ptr = self.decryptor_template.kerberos_ticket_struct)
 		self.current_ticket_type = None
 		self.credentials.append(self.current_cred)
 	
